#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Este script hace que el cliente se ponga a escuchar en el puerto e IP Multicast especificados en el único parámetro (cuya sintaxis es "Port:IP"), con un proceso "uftpd".
Posteriormente, el servidor puede hacer una transferencia UFTP a la IP Multicast, o directamente a la IP de un cliente ogLive.
NOTA: La imagen se enviará a la ruta de la caché de los clientes (que actualmente es "/opt/opengnsys/cache").

Paquetes APT requeridos: "uftp" (se puede instalar con "sudo apt install uftp").  

 Parámetros
------------
sys.argv[1] - Parámetros Multicast (en formato "Port:IP")
            - Ejemplo: 9000:239.194.17.2

 Sintaxis
----------
./listenUFTPD.py Port:IP

 Ejemplo
---------
./listenUFTPD.py 9000:239.194.17.2
"""

# --------------------------------------------------------------------------------------------
#                                        IMPORTS
# --------------------------------------------------------------------------------------------

import os
import sys
import subprocess


# --------------------------------------------------------------------------------------------
#                                        VARIABLES
# --------------------------------------------------------------------------------------------

script_name = os.path.basename(__file__)
cache_path = '/opt/opengnsys/cache'
log_file = '/tmp/uftpd.log'


# --------------------------------------------------------------------------------------------
#                                        FUNCTIONS
# --------------------------------------------------------------------------------------------


def show_help():
    """ Imprime la ayuda, cuando se ejecuta el script con el parámetro "help".
    """
    help_text = f"""
        Sintaxis: {script_name} Port:IP
        Ejemplo: {script_name} 9000:239.194.17.2
        """
    print(help_text)


def check_params():
    """ Comprueba que se haya enviado la cantidad correcta de parámetros, y en el formato correcto.
        Si no es así, muestra un mensaje de error, y sale del script.
        LLama a la función "show_help" cuando se ejecuta el script con el parámetro "help".
    """
    # Si se ejecuta el script con el parámetro "help", se muestra la ayuda, y se sale del script:
    if len(sys.argv) == 2 and sys.argv[1] == "help":
        show_help()
        sys.exit(0)
    # Si se ejecuta la función con más o menos de 1 parámetro, se muestra un mensaje de error, y se sale del script:
    elif len(sys.argv) != 2:
        print(f"{script_name} Error: Formato incorrecto: Se debe especificar 1 parámetro (port:ip)")
        sys.exit(1)
    # Si en el parámetro no hay 2 elementos (separados por ":"), se muestra un mensaje de error, y se sale del script:
    param_list = sys.argv[1].split(':')
    if len(param_list) != 2:
        print(f"{script_name} Error: Datos Multicast incorrectos: \"{sys.argv[1]}\" (se debe especificar \"puerto:ip\")")
        sys.exit(2)



# --------------------------------------------------------------------------------------------
#                                          MAIN
# --------------------------------------------------------------------------------------------


def main():
    """
    """
    # Evaluamos si se ha enviado la cantidad correcta de parámetros, y en el formato correcto:
    check_params()

    # Almacenamos los elementos del parámetro en variables (su formato es "puerto:ip"):
    param_list = sys.argv[1].split(':')
    port, ip = param_list

    # Creamos una lista con el comando a enviar (esto es requerido por la función "subprocess.run"), e impimimos el comando con espacios:
    splitted_cmd = f"uftpd -M {ip} -p {port} -L {log_file} -D {cache_path} -E -K rsa:1024".split()

    print(f"Sending command: {' '.join(splitted_cmd)}")

    # Ejecutamos el comando en el sistema, e imprimimos el resultado:
    try:
        result = subprocess.run(splitted_cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(f"ReturnCode: {result.returncode}")
    except subprocess.CalledProcessError as error:
        print(f"ReturnCode: {error.returncode}")
        print(f"Error Output: {error.stderr.decode()}")
    except Exception as error:
        print(f"Se ha producido un error inesperado: {error}")



# --------------------------------------------------------------------------------------------

if __name__ == "__main__":
    main()

# --------------------------------------------------------------------------------------------
